#ifndef QT_FEATURES_Graphs_src_qtgraphs_config_h_H
#define QT_FEATURES_Graphs_src_qtgraphs_config_h_H

#define QT_FEATURE_graphs_2d 1

#define QT_FEATURE_graphs_2d_area 1

#define QT_FEATURE_graphs_2d_bar 1

#define QT_FEATURE_graphs_2d_donut_pie 1

#define QT_FEATURE_graphs_2d_line 1

#define QT_FEATURE_graphs_2d_scatter 1

#define QT_FEATURE_graphs_2d_spline 1

#define QT_FEATURE_graphs_2d_custom 1

#define QT_FEATURE_graphs_3d 1

#define QT_FEATURE_graphs_3d_bars3d 1

#define QT_FEATURE_graphs_3d_scatter3d 1

#define QT_FEATURE_graphs_3d_surface3d 1


#endif // QT_FEATURES_Graphs_src_qtgraphs_config_h_H
